# pdex.py: arrange names of pandas.DataFrame.columns
  # unicode names -> ascii names, or ascii names -> unicode names
import sys
import inspect

# arrange DataFrame.columns' names before and after DataFrame.query()
def query2(self, asc_col, x, inplace=False, **kwargs):
    org_col = list(self.columns)
    if len(org_col) != len(asc_col):
        sys.stderr.write("Error in 'query2': 1st parameter(%s) "
          "is undue length!\n" % asc_col)
        return False
    undo_dict = dict(zip(asc_col, org_col))
    self.columns = asc_col
    if inplace == True:
        self.query(x, inplace=True, **kwargs)
        self.rename(columns = undo_dict, inplace=True)
        res = None
    else:
        res = self.query(x, inplace=False, **kwargs)
        if res.__class__.__name__ in ['DataFrame', 'DplyFrame']:
            res = res.rename(columns = undo_dict)
        elif res.__class__.__name__ == 'Series' and \
             res.name in undo_dict.keys():
            res.name = undo_dict[res.name]
        self.columns = org_col
    return res

# return the source code for adding 'query2'
def add_query2(pd_name = "pd"):
    src = inspect.getsource(query2)
    src = "\n" + src + "\n" + pd_name + ".DataFrame.query2 = query2"
    return src

## --------

# global variables in this module
asc_col = None  # ascii names of columns
org_col = None  # original names of columns
undo_dict = None  # dictorary (ascii -> original)

# transform names of columns into ascii names
  # dtf: DataFrame, acol: list of ascii names
  # return: True in success, otherwise False
def to_asc(dtf, acol):
    if dtf.__class__.__name__ not in ['DataFrame', 'DplyFrame']:
        sys.stderr.write("Error in 'to_asc': the 1st parameter "
          "is not DataFrame|DplyFrame!\n")
        return False
    global asc_col, org_col, undo_dict
    asc_col = acol
    org_col = list(dtf.columns)
    if len(org_col) != len(asc_col):
        sys.stderr.write("Error in 'to_asc': 2nd parameter(%s) "
          "is undue length!\n" % asc_col)
        return False
    undo_dict = dict(zip(asc_col, org_col))
    dtf.columns = asc_col
    return True

# transform ascii names into original names
  # args: an object(DataFrame or Series), or multiple objects
  # return: True in success, otherwise False (in single argument)
  # return: the list of True or False (in multiple arguments)
def to_org(*args):
      # subfunction when 'xobj' is DataFrame or Series (not list)
    def to_org1(xobj):
        if xobj.__class__.__name__ not in ['DataFrame',
          'DplyFrame', 'Series']:
            return False
        if xobj.__class__.__name__ in ['DataFrame', 'DplyFrame']:
            if list(xobj.columns) == asc_col:
                xobj.columns = org_col
            else:
                xobj.rename(columns = undo_dict, inplace=True)
        elif xobj.name in undo_dict.keys():
            xobj.name = undo_dict[xobj.name]
        return True
      # processing a argument or multiple arguments
    global asc_col, org_col, undo_dict
    if org_col is None:
        sys.stderr.write("Error in 'to_org': can't find the names "
          "of original columns!\n")
        return False
    if len(args) == 1:
        return to_org1(args[0])
    else:
        res = list()
        for i in range(len(args)):
            res.append( to_org1(args[i]) )
        return res

# clear the three global variables
def clear_vars():
    global asc_col, org_col, undo_dict
    asc_col = None
    org_col = None
    undo_dict = None

# alias of methods
do = to_asc
undo = to_org

